@@ -1,28 +1,37 @@
 package Data::Printer::Filter;
 use strict;
 use warnings;
-use Clone::PP qw(clone);
-require Data::Printer;
+use Data::Printer::Common;
 
-my %_filters_for = ();
+require Carp;
+#require Data::Printer;
+
+my %_filters_for   = ();
+my %_extras_for    = ();
 
 sub import {
     my $caller = caller;
-    my $id = Data::Printer::_object_id( \$caller );
-
-    my %properties = ();
+    my $id = Data::Printer::Common::_object_id( \$caller );
 
     my $filter = sub {
-        my ($type, $code) = @_;
+        my ($type, $code, $extra) = @_;
+
+        Carp::croak( "syntax: filter 'Class', sub { ... }" )
+          unless $type and $code and ref $code eq 'CODE';
+
+        if ($extra) {
+            Carp::croak( 'extra filter field must be a hashref' )
+                unless ref $extra and ref $extra eq 'HASH';
+
+            $_extras_for{$id}{$type} = $extra;
+        }
+        else {
+            $_extras_for{$id}{$type} = {};
+        }
 
         unshift @{ $_filters_for{$id}{$type} }, sub {
-            my ($item, $p) = @_;
-
-            # send our closured %properties var instead
-            # so newline(), indent(), etc can work it
-            %properties = %{ clone $p };
-            delete $properties{filters}; # no need to rework filters
-            $code->($item, \%properties);
+            my ($item, $ddp) = @_;
+            $code->($item, $ddp);
         };
     };
 
@@ -30,28 +39,35 @@ sub import {
         return $_filters_for{$id};
     };
 
+    my $extras = sub {
+        return $_extras_for{$id};
+    };
+
     my $newline = sub {
-        return ${$properties{_linebreak}} . (' ' x $properties{_current_indent});
+#TODO FIXME        return $properties{_linebreak} . (' ' x $properties{_current_indent});
     };
 
     my $indent = sub {
-        $properties{_current_indent} += $properties{indent};
-        $properties{_depth}++;
+#TODO FIXME        $properties{_current_indent} += $properties{indent};
+#TODO FIXME        $properties{_depth}++;
         return;
     };
 
     my $outdent = sub {
-        $properties{_current_indent} -= $properties{indent};
-        $properties{_depth}--;
+#TODO FIXME        $properties{_current_indent} -= $properties{indent};
+#TODO FIXME        $properties{_depth}--;
         return;
     };
 
-    my $imported = sub (\[@$%&];%) {
+    my $imported_p = sub (\[@$%&];%) {
         my ($item, $p) = @_;
-        require Data::Printer;
-        return Data::Printer::p( $item, %properties );
+#TODO FIXME        return Data::Printer::p( $item, %properties );
     };
 
+    my $imported_np = sub (\[@$%&];%) {
+        my ($item, $p) = @_;
+#TODO FIXME        return Data::Printer::np( $item, %properties );
+    };
     {
         no strict 'refs';
         *{"$caller\::filter"}  = $filter;
@@ -59,9 +75,11 @@ sub import {
         *{"$caller\::outdent"} = $outdent;
         *{"$caller\::newline"} = $newline;
 
-        *{"$caller\::p"} = $imported;
+        *{"$caller\::np"} = $imported_np;
+        *{"$caller\::p"} = $imported_p;
 
-        *{"$caller\::_filter_list"} = $filters;
+        *{"$caller\::_filter_list"}   = $filters;
+        *{"$caller\::_extra_options"} = $extras;
     }
 };
 
@@ -87,7 +105,7 @@ Create your filter module:
   filter 'SCALAR', sub {
       my ($ref, $properties) = @_;
       my $val = $$ref;
-      
+
       if ($val > 100) {
           return 'too big!!';
       }
@@ -191,6 +209,12 @@ You can use this to throw some data structures back at C<Data::Printer>
 and use the results in your own return string - like when manipulating
 hashes or arrays.
 
+=head2 np()
+
+This is the same as C<Data::Printer>'s np().  You can use this to throw some
+data structures back at C<Data::Printer> and use the results in your own return
+string - like when manipulating hashes or arrays.
+
 =head2 newline()
 
 This helper returns a string using the linebreak as specified by the
@@ -265,12 +289,36 @@ L<Data::Printer::Filter::DateTime> pretty-prints several date
 and time objects (not just DateTime) for you on the fly, including
 duration/delta objects!
 
+=head2 Digest
+
+L<Data::Printer::Filter::Digest> displays a string containing the
+hash of the actual message digest instead of the object. Works on
+C<Digest::MD5>, C<Digest::SHA>, any digest class that inherits from
+C<Digest::base> and some others that implement their own thing!
+
+=head2 ClassicRegex
+
+L<Data::Printer::Filter::ClassicRegex> changes the way Data::Printer
+dumps regular expressions, doing it the classic C<qr//> way that got
+popular in C<Data::Dumper>.
+
+=head2 JSON
+
+L<Data::Printer::Filter::JSON>, by Nuba Princigalli, lets you see
+your JSON structures replacing boolean objects with simple C<true/false>
+strings!
+
 =head2 URIs
 
 L<Data::Printer::Filter::URI> filters through several L<URI> manipulation
 classes and displays the URI as a colored string. A very nice addition
 by Stanislaw Pusep (SYP).
 
+=head2 Perl Data Language (PDL)
+
+L<Data::Printer::Filter::PDL>, by Zakariyya Mughal, lets you quickly see
+the relevant contents of a PDL variable.
+
 =head1 USING MORE THAN ONE FILTER FOR THE SAME TYPE/CLASS
 
 As of version 0.13, standalone filters let you stack together
@@ -279,7 +327,7 @@ called in order, until one of them returns a string. This lets
 you have several filters inspecting the same given value until
 one of them decides to actually treat it somehow.
 
-If your filter catched a value and you don't want to treat it,
+If your filter caught a value and you don't want to treat it,
 simply return and the next filter will be called. If there are no
 other filters for that particular class or type available, the
 standard Data::Printer calls will be used.
@@ -319,5 +367,3 @@ Copyright 2011 Breno G. de Oliveira C<< <garu at cpan.org> >>. All rights reserv
 
 This module is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself. See L<perlartistic>.
-
-
